@@ -44,7 +44,8 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 #include <lib/redeyereduction/redeyereductiontool.h>
 #include <lib/gwenviewconfig.h>
 #include <lib/imageview.h>
-#include <lib/resizeimageoperation.h>
+#include <lib/resize/resizeimageoperation.h>
+#include <lib/resize/resizeimagedialog.h>
 #include <lib/transformimageoperation.h>
 
 namespace Gwenview {
@@ -96,10 +97,12 @@ struct ImageOpsContextManagerItem::Private {
 		mResizeAction = edit->addAction("resize",that, SLOT(resizeImage()) );
 		mResizeAction->setText(i18n("Resize"));
 		mResizeAction->setIcon(KIcon("transform-scale"));
+		mResizeAction->setShortcut(Qt::SHIFT + Qt::Key_R);
 
 		mCropAction = edit->addAction("crop",that, SLOT(crop()));
 		mCropAction->setText(i18n("Crop"));
 		mCropAction->setIcon(KIcon("transform-crop-and-resize"));
+		mCropAction->setShortcut(Qt::SHIFT + Qt::Key_C);
 
 		mRedEyeReductionAction = edit->addAction("red_eye_reduction",that, SLOT(startRedEyeReduction()) );
 		mRedEyeReductionAction->setText(i18n("Red Eye Reduction"));
@@ -169,7 +172,7 @@ void ImageOpsContextManagerItem::updateActions() {
 		// Since we only support image operations on one image for now,
 		// disable actions if several images are selected and the document
 		// view is not visible.
-		if (contextManager()->selection().count() != 1) {
+		if (contextManager()->selectedFileItemList().count() != 1) {
 			canModify = false;
 		}
 	}
@@ -216,22 +219,12 @@ void ImageOpsContextManagerItem::resizeImage() {
 	}
 	Document::Ptr doc = DocumentFactory::instance()->load(contextManager()->currentUrl());
 	doc->startLoadingFullImage();
-	int size = GwenviewConfig::imageResizeLastSize();
-	if (size == -1) {
-		size = qMax(doc->width(), doc->height());
-	}
-	bool ok = false;
-	size = KInputDialog::getInteger(
-		i18n("Image Resizing"),
-		i18n("Enter the new size of the image:"),
-		size, 0, 100000, 10 /* step */,
-		&ok,
-		d->mMainWindow);
-	if (!ok) {
-		return;
-	}
-	GwenviewConfig::setImageResizeLastSize(size);
-	ResizeImageOperation* op = new ResizeImageOperation(size);
+    ResizeImageDialog dialog(d->mMainWindow);
+    dialog.setOriginalSize(doc->size());
+    if (!dialog.exec()) {
+        return;
+    }
+	ResizeImageOperation* op = new ResizeImageOperation(dialog.size());
 	applyImageOperation(op);
 }
 
