@@ -71,6 +71,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #include "imageopscontextmanageritem.h"
 #include "infocontextmanageritem.h"
 #ifdef KIPI_FOUND
+#include "kipiexportaction.h"
 #include "kipiinterface.h"
 #endif
 #ifndef GWENVIEW_SEMANTICINFO_BACKEND_NONE
@@ -171,10 +172,15 @@ struct MainWindow::Private {
 	KAction* mGoUpAction;
 	KAction* mGoToPreviousAction;
 	KAction* mGoToNextAction;
+	KAction* mGoToFirstAction;
+	KAction* mGoToLastAction;
 	KToggleAction* mToggleSideBarAction;
 	KToggleFullScreenAction* mFullScreenAction;
 	KAction* mToggleSlideShowAction;
 	KToggleAction* mShowMenuBarAction;
+#ifdef KIPI_FOUND
+	KIPIExportAction* mKIPIExportAction;
+#endif
 
 	SortedDirModel* mDirModel;
 	ContextManager* mContextManager;
@@ -256,8 +262,6 @@ struct MainWindow::Private {
 		// Connect thumbnail view
 		connect(mThumbnailView, SIGNAL(indexActivated(const QModelIndex&)),
 			mWindow, SLOT(slotThumbnailViewIndexActivated(const QModelIndex&)) );
-		connect(mThumbnailView->model(), SIGNAL(dataChanged(const QModelIndex&, const QModelIndex&)),
-			mWindow, SLOT(slotDataChanged(const QModelIndex&, const QModelIndex&)) );
 		connect(mThumbnailView->selectionModel(), SIGNAL(selectionChanged(const QItemSelection&, const QItemSelection&)),
 			mWindow, SLOT(slotSelectionChanged()) );
 
@@ -359,6 +363,15 @@ struct MainWindow::Private {
 		mGoToNextAction->setToolTip(i18n("Go to Next Image"));
 		mGoToNextAction->setShortcut(Qt::Key_Space);
 
+		mGoToFirstAction = view->addAction("go_first",mWindow, SLOT(goToFirst()));
+		mGoToFirstAction->setText(i18nc("@action Go to first image", "First"));
+		mGoToFirstAction->setToolTip(i18n("Go to First Image"));
+		mGoToFirstAction->setShortcut(Qt::Key_Home);
+
+		mGoToLastAction = view->addAction("go_last",mWindow, SLOT(goToLast()));
+		mGoToLastAction->setText(i18nc("@action Go to last image", "Last"));
+		mGoToLastAction->setToolTip(i18n("Go to Last Image"));
+		mGoToLastAction->setShortcut(Qt::Key_End);
 
 		mGoUpAction = view->addAction(KStandardAction::Up,mWindow, SLOT(goUp()));
 
@@ -388,6 +401,11 @@ struct MainWindow::Private {
 
 		view->addAction(KStandardAction::ConfigureToolbars,mWindow,
 			SLOT(configureToolbars()));
+
+#ifdef KIPI_FOUND
+		mKIPIExportAction = new KIPIExportAction(mWindow);
+		actionCollection->addAction("kipi_export", mKIPIExportAction);
+#endif
 	}
 
 	void setupUndoActions() {
@@ -417,8 +435,7 @@ struct MainWindow::Private {
 	void setupContextManager() {
 		KActionCollection* actionCollection = mWindow->actionCollection();
 
-		mContextManager = new ContextManager(mWindow);
-		mContextManager->setDirModel(mDirModel);
+		mContextManager = new ContextManager(mDirModel, mThumbnailView->selectionModel(), mWindow);
 
 		// Create context manager items
 		FolderViewContextManagerItem* folderViewItem = new FolderViewContextManagerItem(mContextManager);
@@ -450,19 +467,19 @@ struct MainWindow::Private {
 			new ImageOpsContextManagerItem(mContextManager, mWindow);
 		mContextManager->addItem(imageOpsItem);
 
-		FileOpsContextManagerItem* fileOpsItem = new FileOpsContextManagerItem(mContextManager, actionCollection);
+		FileOpsContextManagerItem* fileOpsItem = new FileOpsContextManagerItem(mContextManager, mThumbnailView, actionCollection);
 		mContextManager->addItem(fileOpsItem);
 
 		// Fill sidebar
 		SideBarPage* page;
 		page = new SideBarPage(i18n("Folders"));
-		page->setObjectName("folders");
+		page->setObjectName( QLatin1String("folders" ));
 		page->addWidget(folderViewItem->widget());
 		page->layout()->setMargin(0);
 		mSideBar->addPage(page);
 
 		page = new SideBarPage(i18n("Information"));
-		page->setObjectName("information");
+		page->setObjectName( QLatin1String("information" ));
 		page->addWidget(infoItem->widget());
 		#ifndef GWENVIEW_SEMANTICINFO_BACKEND_NONE
 		if (semanticInfoItem) {
@@ -473,7 +490,7 @@ struct MainWindow::Private {
 		mSideBar->addPage(page);
 
 		page = new SideBarPage(i18n("Operations"));
-		page->setObjectName("operations");
+		page->setObjectName( QLatin1String("operations" ));
 		page->addWidget(imageOpsItem->widget());
 		page->addWidget(fileOpsItem->widget());
 		page->addStretch();
@@ -515,6 +532,15 @@ struct MainWindow::Private {
 		return ArchiveUtils::fileItemIsDirOrArchive(item);
 	}
 
+	void goTo(const QModelIndex& index) {
+		if (!index.isValid()) {
+			return;
+		}
+		mThumbnailView->setCurrentIndex(index);
+		mThumbnailView->scrollTo(index);
+	}
+
+
 	void goTo(int offset) {
 		QModelIndex index = currentIndex();
 		index = mDirModel->index(index.row() + offset, 0);
@@ -522,8 +548,7 @@ struct MainWindow::Private {
 			return;
 		}
 		if (!indexIsDirOrArchive(index)) {
-			mThumbnailView->setCurrentIndex(index);
-			mThumbnailView->scrollTo(index);
+			goTo(index);
 		}
 	}
 
@@ -539,8 +564,12 @@ struct MainWindow::Private {
 				break;
 			}
 		}
-		Q_ASSERT(index.isValid());
-		mThumbnailView->setCurrentIndex(index);
+		goTo(index);
+	}
+
+	void goToLastDocument() {
+		QModelIndex index = mDirModel->index(mDirModel->rowCount() - 1, 0);
+		goTo(index);
 	}
 
 	void spreadCurrentDirUrl(const KUrl& url) {
@@ -684,28 +713,8 @@ struct MainWindow::Private {
 	}
 
 	void updateContextDependentComponents() {
-		// Gather info
-		KFileItemList selectedItemList;
-		KUrl url;
-		if (mCurrentPageId != StartPageId) {
-			QItemSelection selection = mThumbnailView->selectionModel()->selection();
-
-			Q_FOREACH(const QModelIndex& index, selection.indexes()) {
-				selectedItemList << mDirModel->itemForIndex(index);
-			}
-
-			// At least add current url if it's valid (it may not be in
-			// selectedItemList if we are viewing a non-browsable url, for example
-			// using http protocol)
-			url = currentUrl();
-			if (selectedItemList.isEmpty() && url.isValid()) {
-				KFileItem item(KFileItem::Unknown, KFileItem::Unknown, url);
-				selectedItemList << item;
-			}
-		}
-
-		// Update
-		mContextManager->setContext(url, selectedItemList);
+		KUrl url = currentUrl();
+		mContextManager->setCurrentUrl(url);
 		mSaveBar->setCurrentUrl(url);
 		mSlideShow->setCurrentUrl(url);
 	}
@@ -794,6 +803,7 @@ d(new MainWindow::Private)
 
 #ifdef KIPI_FOUND
 	d->mKIPIInterface = new KIPIInterface(this);
+	d->mKIPIExportAction->setKIPIInterface(d->mKIPIInterface);
 #endif
 	setAutoSaveSettings();
 }
@@ -1058,7 +1068,7 @@ void MainWindow::slotPartCompleted() {
 	}
 
 	KUrl dirUrl = url;
-	dirUrl.setFileName("");
+	dirUrl.setFileName(QString());
 	if (dirUrl.equals(d->mDirModel->dirLister()->url(), KUrl::CompareWithoutTrailingSlash)) {
 		QModelIndex index = d->mDirModel->indexForUrl(url);
 		if (index.isValid()) {
@@ -1105,32 +1115,6 @@ void MainWindow::slotSelectionChanged() {
 }
 
 
-void MainWindow::slotDataChanged(const QModelIndex& topLeft, const QModelIndex& bottomRight) {
-	QModelIndexList selectionList = d->mThumbnailView->selectionModel()->selectedIndexes();
-	if (selectionList.isEmpty()) {
-		return;
-	}
-
-	QModelIndexList changedList;
-	const QAbstractItemModel* model = topLeft.model();
-	for (int row=topLeft.row(); row <= bottomRight.row(); ++row) {
-		changedList << model->index(row, 0);
-	}
-
-	QModelIndexList& shortList = selectionList;
-	QModelIndexList& longList = changedList;
-	if (shortList.length() > longList.length()) {
-		qSwap(shortList, longList);
-	}
-	Q_FOREACH(const QModelIndex& index, shortList) {
-		if (longList.contains(index)) {
-			d->updateContextDependentComponents();
-			return;
-		}
-	}
-}
-
-
 void MainWindow::slotDirModelNewItems() {
 	if (d->mUrlToSelect.isValid()) {
 		d->selectUrlToSelect();
@@ -1163,6 +1147,13 @@ void MainWindow::goToNext() {
 	d->goTo(1);
 }
 
+void MainWindow::goToFirst() {
+	d->goToFirstDocument();
+}
+
+void MainWindow::goToLast() {
+	d->goToLastDocument();
+}
 
 void MainWindow::goToUrl(const KUrl& url) {
 	if (d->mCurrentPageId == ViewPageId) {
@@ -1183,9 +1174,11 @@ void MainWindow::updatePreviousNextActions() {
 
 	QModelIndex prevIndex = d->mDirModel->index(index.row() - 1, 0);
 	d->mGoToPreviousAction->setEnabled(prevIndex.isValid() && !d->indexIsDirOrArchive(prevIndex));
+	d->mGoToFirstAction->setEnabled(d->mGoToPreviousAction->isEnabled());
 
 	QModelIndex nextIndex = d->mDirModel->index(index.row() + 1, 0);
 	d->mGoToNextAction->setEnabled(nextIndex.isValid() && !d->indexIsDirOrArchive(nextIndex));
+	d->mGoToLastAction->setEnabled(d->mGoToNextAction->isEnabled());
 }
 
 
@@ -1485,7 +1478,7 @@ void MainWindow::preloadNextUrl() {
 
 
 QSize MainWindow::sizeHint() const {
-	return QSize(750, 500);
+	return KXmlGuiWindow::sizeHint().expandedTo(QSize(750, 500));
 }
 
 
