@@ -37,9 +37,10 @@ class QRect;
 namespace Gwenview {
 
 class Document;
+class DocumentJob;
 class AbstractDocumentEditor;
 
-class AbstractDocumentImplPrivate;
+struct AbstractDocumentImplPrivate;
 class AbstractDocumentImpl : public QObject {
 	Q_OBJECT
 public:
@@ -54,7 +55,7 @@ public:
 
 	virtual Document::LoadingState loadingState() const = 0;
 
-	virtual bool save(const KUrl&, const QByteArray& /*format*/) { return false; }
+	virtual DocumentJob* save(const KUrl&, const QByteArray& /*format*/) { return 0; }
 
 	virtual AbstractDocumentEditor* editor() { return 0; }
 
@@ -68,6 +69,8 @@ public:
 
 	virtual void stopAnimation() {}
 
+	Document* document() const;
+
 Q_SIGNALS:
 	void imageRectUpdated(const QRect&);
 	void metaInfoLoaded();
@@ -77,7 +80,6 @@ Q_SIGNALS:
 	void editorUpdated();
 
 protected:
-	Document* document() const;
 	void setDocumentImage(const QImage& image);
 	void setDocumentImageSize(const QSize& size);
 	void setDocumentKind(MimeTypeUtils::Kind);
