@@ -39,7 +39,7 @@ class AbstractDocumentViewAdapter;
 class SlideShow;
 class ZoomWidget;
 
-class DocumentViewPrivate;
+struct DocumentViewPrivate;
 
 /**
  * This widget can display various documents, using an instance of
@@ -97,9 +97,12 @@ private Q_SLOTS:
 	void slotZoomChanged(qreal);
 	void slotZoomWidgetChanged(qreal);
 
+	void slotBusyChanged(const KUrl&, bool);
+
+	void slotKeyPressed(Qt::Key key, bool pressed);
 
 private:
-	friend class DocumentViewPrivate;
+	friend struct DocumentViewPrivate;
 	DocumentViewPrivate* const d;
 
 	void createAdapterForDocument();
