@@ -1,7 +1,7 @@
 // vim: set tabstop=4 shiftwidth=4 noexpandtab
 /*
 Gwenview: an image viewer
-Copyright 2007 Aurélien Gâteau <aurelien.gateau@free.fr>
+Copyright 2007 Aurélien Gâteau <agateau@kde.org>
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -56,6 +56,10 @@ public:
 	QImage thumbnail() const;
 	void setThumbnail(const QImage&);
 
+	// Recreate raw data to represent image
+	// Note: thumbnail must be updated separately
+	void setImage(const QImage& image);
+
 	bool load(const QString& file);
 	bool loadFromData(const QByteArray& rawData);
 	/**
