@@ -35,7 +35,7 @@ class QPoint;
 namespace Gwenview {
 
 
-class ScrollToolPrivate;
+struct ScrollToolPrivate;
 class GWENVIEWLIB_EXPORT ScrollTool : public AbstractImageViewTool {
 	Q_OBJECT
 public:
@@ -51,24 +51,16 @@ public:
 	virtual void mouseMoveEvent(QMouseEvent*);
 	virtual void mouseReleaseEvent(QMouseEvent*);
 	virtual void wheelEvent(QWheelEvent*);
-	virtual void keyPressEvent(QKeyEvent*);
-	virtual void keyReleaseEvent(QKeyEvent*);
 
 	virtual void toolActivated();
 	virtual void toolDeactivated();
 
-	void setMouseWheelBehavior(MouseWheelBehavior);
-	MouseWheelBehavior mouseWheelBehavior() const;
-
 Q_SIGNALS:
 	void zoomInRequested(const QPoint&);
 	void zoomOutRequested(const QPoint&);
 	void previousImageRequested();
 	void nextImageRequested();
 
-protected:
-	virtual bool eventFilter(QObject*, QEvent*);
-
 private:
 	ScrollToolPrivate* const d;
 };
