@@ -1,23 +1,26 @@
-/* ============================================================
+/** ===========================================================
+ * @file
  *
- * This file is a part of kipi-plugins project
- * http://www.kipi-plugins.org
+ * This file is a part of digiKam project
+ * <a href="http://www.digikam.org">http://www.digikam.org</a>
  *
- * Date        : 2006-09-15
- * Description : Exiv2 library interface for KDE
- *               Common metadata image information manipulation methods
+ * @date   2006-09-15
+ * @brief  Common metadata image information manipulation methods
  *
- * Copyright (C) 2006-2009 by Gilles Caulier <caulier dot gilles at gmail dot com>
- * Copyright (C) 2006-2009 by Marcel Wiesweg <marcel dot wiesweg at gmx dot de>
+ * @author Copyright (C) 2006-2010 by Gilles Caulier
+ *         <a href="mailto:caulier dot gilles at gmail dot com">caulier dot gilles at gmail dot com</a>
+ * @author Copyright (C) 2006-2010 by Marcel Wiesweg
+ *         <a href="mailto:marcel dot wiesweg at gmx dot de">marcel dot wiesweg at gmx dot de</a>
  *
  * This program is free software; you can redistribute it
  * and/or modify it under the terms of the GNU General
  * Public License as published by the Free Software Foundation;
- * either version 2, or (at your option) any later version.
+ * either version 2, or (at your option)
+ * any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  * GNU General Public License for more details.
  *
  * ============================================================ */
@@ -26,6 +29,7 @@
 
 #include "kexiv2_p.h"
 #include "kexiv2.h"
+#include "rotationmatrix.h"
 
 namespace KExiv2Iface
 {
@@ -41,28 +45,28 @@ bool KExiv2::setImageProgramId(const QString& program, const QString& version) c
         // Set program info into Exif.Image.ProcessingSoftware tag (only available with Exiv2 >= 0.14.0).
 
 #if (EXIV2_TEST_VERSION(0,14,0))
-        d->exifMetadata["Exif.Image.ProcessingSoftware"] = std::string(software.toAscii().constData());
+        d->exifMetadata()["Exif.Image.ProcessingSoftware"] = std::string(software.toAscii().constData());
 #endif
 
         // See B.K.O #142564: Check if Exif.Image.Software already exist. If yes, do not touch this tag.
 
-        if (!d->exifMetadata.empty())
+        if (!d->exifMetadata().empty())
         {
-            Exiv2::ExifData exifData(d->exifMetadata);
+            Exiv2::ExifData exifData(d->exifMetadata());
             Exiv2::ExifKey key("Exif.Image.Software");
             Exiv2::ExifData::iterator it = exifData.findKey(key);
             if (it == exifData.end())
-                d->exifMetadata["Exif.Image.Software"] = std::string(software.toAscii().constData());
+                d->exifMetadata()["Exif.Image.Software"] = std::string(software.toAscii().constData());
         }
 
         // set program info into XMP tags.
 
 #ifdef _XMP_SUPPORT_
 
-        if (!d->xmpMetadata.empty())
+        if (!d->xmpMetadata().empty())
         {
             // Only create Xmp.xmp.CreatorTool if it do not exist.
-            Exiv2::XmpData xmpData(d->xmpMetadata);
+            Exiv2::XmpData xmpData(d->xmpMetadata());
             Exiv2::XmpKey key("Xmp.xmp.CreatorTool");
             Exiv2::XmpData::iterator it = xmpData.findKey(key);
             if (it == xmpData.end())
@@ -75,11 +79,11 @@ bool KExiv2::setImageProgramId(const QString& program, const QString& version) c
 
         // Set program info into IPTC tags.
 
-        d->iptcMetadata["Iptc.Application2.Program"]        = std::string(program.toAscii().constData());
-        d->iptcMetadata["Iptc.Application2.ProgramVersion"] = std::string(version.toAscii().constData());
+        d->iptcMetadata()["Iptc.Application2.Program"]        = std::string(program.toAscii().constData());
+        d->iptcMetadata()["Iptc.Application2.ProgramVersion"] = std::string(version.toAscii().constData());
         return true;
     }
-    catch( Exiv2::Error &e )
+    catch( Exiv2::Error& e )
     {
         d->printExiv2ExceptionError("Cannot set Program identity into image using Exiv2 ", e);
     }
@@ -95,15 +99,15 @@ QSize KExiv2::getImageDimensions() const
 
         // Try to get Exif.Photo tags
 
-        Exiv2::ExifData exifData(d->exifMetadata);
+        Exiv2::ExifData exifData(d->exifMetadata());
         Exiv2::ExifKey key("Exif.Photo.PixelXDimension");
         Exiv2::ExifData::iterator it = exifData.findKey(key);
-        if (it != exifData.end())
+        if (it != exifData.end() && it->count())
             width = it->toLong();
 
         Exiv2::ExifKey key2("Exif.Photo.PixelYDimension");
         Exiv2::ExifData::iterator it2 = exifData.findKey(key2);
-        if (it2 != exifData.end())
+        if (it2 != exifData.end() && it2->count())
             height = it2->toLong();
 
         if (width != -1 && height != -1)
@@ -116,12 +120,12 @@ QSize KExiv2::getImageDimensions() const
 
         Exiv2::ExifKey key3("Exif.Image.ImageWidth");
         Exiv2::ExifData::iterator it3 = exifData.findKey(key3);
-        if (it3 != exifData.end())
+        if (it3 != exifData.end() && it3->count())
             width = it3->toLong();
 
         Exiv2::ExifKey key4("Exif.Image.ImageLength");
         Exiv2::ExifData::iterator it4 = exifData.findKey(key4);
-        if (it4 != exifData.end())
+        if (it4 != exifData.end() && it4->count())
             height = it4->toLong();
 
         if (width != -1 && height != -1)
@@ -168,7 +172,7 @@ QSize KExiv2::getImageDimensions() const
 #endif // _XMP_SUPPORT_
 
     }
-    catch( Exiv2::Error &e )
+    catch( Exiv2::Error& e )
     {
         d->printExiv2ExceptionError("Cannot parse image dimensions tag using Exiv2 ", e);
     }
@@ -186,10 +190,10 @@ bool KExiv2::setImageDimensions(const QSize& size, bool setProgramName) const
         // Set Exif values.
 
         // NOTE: see B.K.O #144604: need to cast to record an unsigned integer value.
-        d->exifMetadata["Exif.Image.ImageWidth"]      = static_cast<uint32_t>(size.width());
-        d->exifMetadata["Exif.Image.ImageLength"]     = static_cast<uint32_t>(size.height());
-        d->exifMetadata["Exif.Photo.PixelXDimension"] = static_cast<uint32_t>(size.width());
-        d->exifMetadata["Exif.Photo.PixelYDimension"] = static_cast<uint32_t>(size.height());
+        d->exifMetadata()["Exif.Image.ImageWidth"]      = static_cast<uint32_t>(size.width());
+        d->exifMetadata()["Exif.Image.ImageLength"]     = static_cast<uint32_t>(size.height());
+        d->exifMetadata()["Exif.Photo.PixelXDimension"] = static_cast<uint32_t>(size.width());
+        d->exifMetadata()["Exif.Photo.PixelYDimension"] = static_cast<uint32_t>(size.height());
 
         // Set Xmp values.
 
@@ -204,7 +208,7 @@ bool KExiv2::setImageDimensions(const QSize& size, bool setProgramName) const
 
         return true;
     }
-    catch( Exiv2::Error &e )
+    catch( Exiv2::Error& e )
     {
         d->printExiv2ExceptionError("Cannot set image dimensions using Exiv2 ", e);
     }
@@ -216,12 +220,12 @@ KExiv2::ImageOrientation KExiv2::getImageOrientation() const
 {
     try
     {
-        Exiv2::ExifData exifData(d->exifMetadata);
+        Exiv2::ExifData exifData(d->exifMetadata());
         Exiv2::ExifData::iterator it;
         long orientation;
         ImageOrientation imageOrient = ORIENTATION_NORMAL;
 
-        // Because some camera set a wrong standard exif orientation tag, 
+        // Because some camera set a wrong standard exif orientation tag,
         // We need to check makernote tags in first!
 
         // -- Minolta Cameras ----------------------------------
@@ -229,10 +233,10 @@ KExiv2::ImageOrientation KExiv2::getImageOrientation() const
         Exiv2::ExifKey minoltaKey1("Exif.MinoltaCs7D.Rotation");
         it = exifData.findKey(minoltaKey1);
 
-        if (it != exifData.end())
+        if (it != exifData.end() && it->count())
         {
             orientation = it->toLong();
-            kDebug(51003) << "Orientation => Exif.MinoltaCs7D.Rotation => " << (int)orientation << endl;
+            kDebug() << "Orientation => Exif.MinoltaCs7D.Rotation => " << (int)orientation;
             switch(orientation)
             {
                 case 76:
@@ -248,10 +252,10 @@ KExiv2::ImageOrientation KExiv2::getImageOrientation() const
         Exiv2::ExifKey minoltaKey2("Exif.MinoltaCs5D.Rotation");
         it = exifData.findKey(minoltaKey2);
 
-        if (it != exifData.end())
+        if (it != exifData.end() && it->count())
         {
             orientation = it->toLong();
-            kDebug(51003) << "Orientation => Exif.MinoltaCs5D.Rotation => " << (int)orientation << endl;
+            kDebug() << "Orientation => Exif.MinoltaCs5D.Rotation => " << (int)orientation;
             switch(orientation)
             {
                 case 76:
@@ -269,10 +273,10 @@ KExiv2::ImageOrientation KExiv2::getImageOrientation() const
         Exiv2::ExifKey keyStd("Exif.Image.Orientation");
         it = exifData.findKey(keyStd);
 
-        if (it != exifData.end())
+        if (it != exifData.end() && it->count())
         {
             orientation = it->toLong();
-            kDebug(51003) << "Orientation => Exif.Image.Orientation => " << (int)orientation << endl;
+            kDebug() << "Orientation => Exif.Image.Orientation => " << (int)orientation;
             return (ImageOrientation)orientation;
         }
 
@@ -287,7 +291,7 @@ KExiv2::ImageOrientation KExiv2::getImageOrientation() const
             orientation = str.toLong(&ok);
             if (ok)
             {
-                kDebug(51003) << "Orientation => Xmp.tiff.Orientation => " << (int)orientation << endl;
+                kDebug() << "Orientation => Xmp.tiff.Orientation => " << (int)orientation;
                 return (ImageOrientation)orientation;
             }
         }
@@ -295,7 +299,7 @@ KExiv2::ImageOrientation KExiv2::getImageOrientation() const
 #endif // _XMP_SUPPORT_
 
     }
-    catch( Exiv2::Error &e )
+    catch( Exiv2::Error& e )
     {
         d->printExiv2ExceptionError("Cannot parse Exif Orientation tag using Exiv2 ", e);
     }
@@ -312,14 +316,14 @@ bool KExiv2::setImageOrientation(ImageOrientation orientation, bool setProgramNa
     {
         if (orientation < ORIENTATION_UNSPECIFIED || orientation > ORIENTATION_ROT_270)
         {
-            kDebug(51003) << "Image orientation value is not correct!" << endl;
+            kDebug() << "Image orientation value is not correct!";
             return false;
         }
 
         // Set Exif values.
 
-        d->exifMetadata["Exif.Image.Orientation"] = static_cast<uint16_t>(orientation);
-        kDebug(51003) << "Exif.Image.Orientation tag set to: " << (int)orientation << endl;
+        d->exifMetadata()["Exif.Image.Orientation"] = static_cast<uint16_t>(orientation);
+        kDebug() << "Exif.Image.Orientation tag set to: " << (int)orientation;
 
         // Set Xmp values.
 
@@ -329,32 +333,43 @@ bool KExiv2::setImageOrientation(ImageOrientation orientation, bool setProgramNa
 
 #endif // _XMP_SUPPORT_
 
-        // -- Minolta Cameras ----------------------------------
+        // -- Minolta/Sony Cameras ----------------------------------
 
-        // Minolta camera store image rotation in Makernote.
-        // We remove these information to prevent duplicate values. 
+        // Minolta and Sony camera store image rotation in Makernote.
+        // We remove these information to prevent duplicate values.
 
         Exiv2::ExifData::iterator it;
 
         Exiv2::ExifKey minoltaKey1("Exif.MinoltaCs7D.Rotation");
-        it = d->exifMetadata.findKey(minoltaKey1);
-        if (it != d->exifMetadata.end())
+        it = d->exifMetadata().findKey(minoltaKey1);
+        if (it != d->exifMetadata().end())
         {
-            d->exifMetadata.erase(it);
-            kDebug(51003) << "Removing Exif.MinoltaCs7D.Rotation tag" << endl;
+            d->exifMetadata().erase(it);
+            kDebug() << "Removing Exif.MinoltaCs7D.Rotation tag";
         }
 
         Exiv2::ExifKey minoltaKey2("Exif.MinoltaCs5D.Rotation");
-        it = d->exifMetadata.findKey(minoltaKey2);
-        if (it != d->exifMetadata.end())
+        it = d->exifMetadata().findKey(minoltaKey2);
+        if (it != d->exifMetadata().end())
         {
-            d->exifMetadata.erase(it);
-            kDebug(51003) << "Removing Exif.MinoltaCs5D.Rotation tag" << endl;
+            d->exifMetadata().erase(it);
+            kDebug() << "Removing Exif.MinoltaCs5D.Rotation tag";
+        }
+
+        // -- Exif embedded thumbnail ----------------------------------
+
+        Exiv2::ExifKey thumbKey("Exif.Thumbnail.Orientation");
+        it = d->exifMetadata().findKey(thumbKey);
+        if (it != d->exifMetadata().end() && it->count())
+        {
+            RotationMatrix operation((KExiv2Iface::KExiv2::ImageOrientation)it->toLong());
+            operation *= orientation;
+            (*it) = static_cast<uint16_t>(operation.exifOrientation());
         }
 
         return true;
     }
-    catch( Exiv2::Error &e )
+    catch( Exiv2::Error& e )
     {
         d->printExiv2ExceptionError("Cannot set Exif Orientation tag using Exiv2 ", e);
     }
@@ -366,71 +381,71 @@ KExiv2::ImageColorWorkSpace KExiv2::getImageColorWorkSpace() const
 {
     // Check Exif values.
 
-    long colorSpace = 0;
-
-    if (getExifTagLong("Exif.Photo.ColorSpace", colorSpace))
+    long exifColorSpace = -1;
+    if (!getExifTagLong("Exif.Photo.ColorSpace", exifColorSpace))
     {
-        switch (colorSpace)
-        {
-            case 1:
-            {
-                return WORKSPACE_SRGB;
-                break;
-            }
-            case 2:
-            {
-                return WORKSPACE_ADOBERGB;
-                break;
-            }
-            case 65535:
-            {
-                // Nikon camera set Exif.Photo.ColorSpace to uncalibrated and 
-                // Exif.Nikon3.ColorMode to "MODE2" when users work in AdobRGB color space.
-                if (getExifTagString("Exif.Nikon3.ColorMode").contains("MODE2"))
-                    return WORKSPACE_ADOBERGB;
-
-                // TODO : add more Makernote parsing here ...
-
-                return WORKSPACE_UNCALIBRATED;
-                break;
-            }
-        }
-    }
-
-    // Check Xmp values.
-
 #ifdef _XMP_SUPPORT_
-
-    colorSpace  = 0;
-    bool ok     = false;
-    QString str = getXmpTagString("Xmp.exif.ColorSpace");
-    if (!str.isEmpty())
-    {
-        colorSpace = str.toLong(&ok);
-        if (ok)
-        {
-            switch (colorSpace)
-            {
-                case 1:
-                {
-                    return WORKSPACE_SRGB;
-                    break;
-                }
-                case 2:
-                {
-                    return WORKSPACE_ADOBERGB;
-                    break;
-                }
-                case 65535:
-                {
-                    return WORKSPACE_UNCALIBRATED;
-                    break;
-                }
-            }
-        }
+        QVariant var = getXmpTagVariant("Xmp.exif.ColorSpace");
+        if (!var.isNull())
+            exifColorSpace = var.toInt();
+#endif // _XMP_SUPPORT_
     }
 
-#endif // _XMP_SUPPORT_
+    if (exifColorSpace == 1)
+        return WORKSPACE_SRGB; // as specified by standard
+    else if (exifColorSpace == 2)
+        return WORKSPACE_ADOBERGB; // not in the standard!
+    else
+    {
+        if (exifColorSpace == 65535)
+        {
+            // A lot of cameras set the Exif.Iop.InteroperabilityIndex,
+            // as documented for ExifTool
+            QString interopIndex = getExifTagString("Exif.Iop.InteroperabilityIndex");
+            if (!interopIndex.isNull())
+            {
+                if (interopIndex == "R03")
+                    return WORKSPACE_ADOBERGB;
+                else if (interopIndex == "R98")
+                    return WORKSPACE_SRGB;
+            }
+        }
+
+        // Note: Text EXIF ColorSpace tag may just not be present (NEF files)
+
+        // Nikon camera set Exif.Photo.ColorSpace to uncalibrated or just skip this field,
+        // then add additional information into the makernotes.
+        // Exif.Nikon3.ColorSpace: 1 => sRGB, 2 => AdobeRGB
+        long nikonColorSpace;
+        if (getExifTagLong("Exif.Nikon3.ColorSpace", nikonColorSpace))
+        {
+            if (nikonColorSpace == 1)
+                return WORKSPACE_SRGB;
+            else if (nikonColorSpace == 2)
+                return WORKSPACE_ADOBERGB;
+        }
+        // Exif.Nikon3.ColorMode is set to "MODE2" for AdobeRGB, but there are sometimes two ColorMode fields
+        // in a NEF, with the first one "COLOR" and the second one "MODE2"; but in this case, ColorSpace (above) was set.
+        if (getExifTagString("Exif.Nikon3.ColorMode").contains("MODE2"))
+            return WORKSPACE_ADOBERGB;
+
+        //TODO: This makernote tag (0x00b4) must be added to libexiv2
+        /*
+        long canonColorSpace;
+        if (getExifTagLong("Exif.Canon.ColorSpace", canonColorSpace))
+        {
+            if (canonColorSpace == 1)
+                return WORKSPACE_SRGB;
+            else if (canonColorSpace == 2)
+                return WORKSPACE_ADOBERGB;
+        }
+        */
+
+        // TODO : add more Makernote parsing here ...
+
+        if (exifColorSpace == 65535)
+            return WORKSPACE_UNCALIBRATED;
+    }
 
     return WORKSPACE_UNSPECIFIED;
 }
@@ -444,7 +459,7 @@ bool KExiv2::setImageColorWorkSpace(ImageColorWorkSpace workspace, bool setProgr
     {
         // Set Exif value.
 
-        d->exifMetadata["Exif.Photo.ColorSpace"] = static_cast<uint16_t>(workspace);
+        d->exifMetadata()["Exif.Photo.ColorSpace"] = static_cast<uint16_t>(workspace);
 
         // Set Xmp value.
 
@@ -456,7 +471,7 @@ bool KExiv2::setImageColorWorkSpace(ImageColorWorkSpace workspace, bool setProgr
 
         return true;
     }
-    catch( Exiv2::Error &e )
+    catch( Exiv2::Error& e )
     {
         d->printExiv2ExceptionError("Cannot set Exif color workspace tag using Exiv2 ", e);
     }
@@ -470,9 +485,9 @@ QDateTime KExiv2::getImageDateTime() const
     {
         // In first, trying to get Date & time from Exif tags.
 
-        if (!d->exifMetadata.empty())
+        if (!d->exifMetadata().empty())
         {
-            Exiv2::ExifData exifData(d->exifMetadata);
+            Exiv2::ExifData exifData(d->exifMetadata());
             {
                 Exiv2::ExifKey key("Exif.Photo.DateTimeOriginal");
                 Exiv2::ExifData::iterator it = exifData.findKey(key);
@@ -481,7 +496,7 @@ QDateTime KExiv2::getImageDateTime() const
                     QDateTime dateTime = QDateTime::fromString(it->toString().c_str(), Qt::ISODate);
                     if (dateTime.isValid())
                     {
-                        kDebug(51003) << "DateTime => Exif.Photo.DateTimeOriginal => " << dateTime << endl;
+                        kDebug() << "DateTime => Exif.Photo.DateTimeOriginal => " << dateTime;
                         return dateTime;
                     }
                 }
@@ -494,7 +509,7 @@ QDateTime KExiv2::getImageDateTime() const
                     QDateTime dateTime = QDateTime::fromString(it->toString().c_str(), Qt::ISODate);
                     if (dateTime.isValid())
                     {
-                        kDebug(51003) << "DateTime => Exif.Photo.DateTimeDigitized => " << dateTime << endl;
+                        kDebug() << "DateTime => Exif.Photo.DateTimeDigitized => " << dateTime;
                         return dateTime;
                     }
                 }
@@ -507,7 +522,7 @@ QDateTime KExiv2::getImageDateTime() const
                     QDateTime dateTime = QDateTime::fromString(it->toString().c_str(), Qt::ISODate);
                     if (dateTime.isValid())
                     {
-                        kDebug(51003) << "DateTime => Exif.Image.DateTime => " << dateTime << endl;
+                        kDebug() << "DateTime => Exif.Image.DateTime => " << dateTime;
                         return dateTime;
                     }
                 }
@@ -518,9 +533,9 @@ QDateTime KExiv2::getImageDateTime() const
 
 #ifdef _XMP_SUPPORT_
 
-        if (!d->xmpMetadata.empty())
+        if (!d->xmpMetadata().empty())
         {
-            Exiv2::XmpData xmpData(d->xmpMetadata);
+            Exiv2::XmpData xmpData(d->xmpMetadata());
             {
                 Exiv2::XmpKey key("Xmp.exif.DateTimeOriginal");
                 Exiv2::XmpData::iterator it = xmpData.findKey(key);
@@ -529,7 +544,7 @@ QDateTime KExiv2::getImageDateTime() const
                     QDateTime dateTime = QDateTime::fromString(it->toString().c_str(), Qt::ISODate);
                     if (dateTime.isValid())
                     {
-                        kDebug(51003) << "DateTime => Xmp.exif.DateTimeOriginal => " << dateTime << endl;
+                        kDebug() << "DateTime => Xmp.exif.DateTimeOriginal => " << dateTime;
                         return dateTime;
                     }
                 }
@@ -542,7 +557,7 @@ QDateTime KExiv2::getImageDateTime() const
                     QDateTime dateTime = QDateTime::fromString(it->toString().c_str(), Qt::ISODate);
                     if (dateTime.isValid())
                     {
-                        kDebug(51003) << "DateTime => Xmp.exif.DateTimeDigitized => " << dateTime << endl;
+                        kDebug() << "DateTime => Xmp.exif.DateTimeDigitized => " << dateTime;
                         return dateTime;
                     }
                 }
@@ -555,7 +570,7 @@ QDateTime KExiv2::getImageDateTime() const
                     QDateTime dateTime = QDateTime::fromString(it->toString().c_str(), Qt::ISODate);
                     if (dateTime.isValid())
                     {
-                        kDebug(51003) << "DateTime => Xmp.photoshop.DateCreated => " << dateTime << endl;
+                        kDebug() << "DateTime => Xmp.photoshop.DateCreated => " << dateTime;
                         return dateTime;
                     }
                 }
@@ -568,7 +583,7 @@ QDateTime KExiv2::getImageDateTime() const
                     QDateTime dateTime = QDateTime::fromString(it->toString().c_str(), Qt::ISODate);
                     if (dateTime.isValid())
                     {
-                        kDebug(51003) << "DateTime => Xmp.xmp.CreateDate => " << dateTime << endl;
+                        kDebug() << "DateTime => Xmp.xmp.CreateDate => " << dateTime;
                         return dateTime;
                     }
                 }
@@ -581,7 +596,7 @@ QDateTime KExiv2::getImageDateTime() const
                     QDateTime dateTime = QDateTime::fromString(it->toString().c_str(), Qt::ISODate);
                     if (dateTime.isValid())
                     {
-                        kDebug(51003) << "DateTime => Xmp.tiff.DateTime => " << dateTime << endl;
+                        kDebug() << "DateTime => Xmp.tiff.DateTime => " << dateTime;
                         return dateTime;
                     }
                 }
@@ -594,7 +609,7 @@ QDateTime KExiv2::getImageDateTime() const
                     QDateTime dateTime = QDateTime::fromString(it->toString().c_str(), Qt::ISODate);
                     if (dateTime.isValid())
                     {
-                        kDebug(51003) << "DateTime => Xmp.xmp.ModifyDate => " << dateTime << endl;
+                        kDebug() << "DateTime => Xmp.xmp.ModifyDate => " << dateTime;
                         return dateTime;
                     }
                 }
@@ -607,7 +622,7 @@ QDateTime KExiv2::getImageDateTime() const
                     QDateTime dateTime = QDateTime::fromString(it->toString().c_str(), Qt::ISODate);
                     if (dateTime.isValid())
                     {
-                        kDebug(51003) << "DateTime => Xmp.xmp.MetadataDate => " << dateTime << endl;
+                        kDebug() << "DateTime => Xmp.xmp.MetadataDate => " << dateTime;
                         return dateTime;
                     }
                 }
@@ -618,9 +633,9 @@ QDateTime KExiv2::getImageDateTime() const
 
         // In third, trying to get Date & time from Iptc tags.
 
-        if (!d->iptcMetadata.empty())
+        if (!d->iptcMetadata().empty())
         {
-            Exiv2::IptcData iptcData(d->iptcMetadata);
+            Exiv2::IptcData iptcData(d->iptcMetadata());
 
             // Try creation Iptc date & time entries.
 
@@ -639,7 +654,7 @@ QDateTime KExiv2::getImageDateTime() const
                     QDateTime dateTime = QDateTime(date, time);
                     if (dateTime.isValid())
                     {
-                        kDebug(51003) << "DateTime => Iptc.Application2.DateCreated => " << dateTime << endl;
+                        kDebug() << "DateTime => Iptc.Application2.DateCreated => " << dateTime;
                         return dateTime;
                     }
                 }
@@ -662,14 +677,14 @@ QDateTime KExiv2::getImageDateTime() const
                     QDateTime dateTime = QDateTime(date, time);
                     if (dateTime.isValid())
                     {
-                        kDebug(51003) << "DateTime => Iptc.Application2.DigitizationDate => " << dateTime << endl;
+                        kDebug() << "DateTime => Iptc.Application2.DigitizationDate => " << dateTime;
                         return dateTime;
                     }
                 }
             }
         }
     }
-    catch( Exiv2::Error &e )
+    catch( Exiv2::Error& e )
     {
         d->printExiv2ExceptionError("Cannot parse Exif date & time tag using Exiv2 ", e);
     }
@@ -695,10 +710,10 @@ bool KExiv2::setImageDateTime(const QDateTime& dateTime, bool setDateTimeDigitiz
         // Reference: http://www.exif.org/Exif2-2.PDF, chapter 4.6.5, table 4, section F.
 
         const std::string &exifdatetime(dateTime.toString(QString("yyyy:MM:dd hh:mm:ss")).toAscii().constData());
-        d->exifMetadata["Exif.Image.DateTime"]         = exifdatetime;
-        d->exifMetadata["Exif.Photo.DateTimeOriginal"] = exifdatetime;
+        d->exifMetadata()["Exif.Image.DateTime"]         = exifdatetime;
+        d->exifMetadata()["Exif.Photo.DateTimeOriginal"] = exifdatetime;
         if(setDateTimeDigitized)
-            d->exifMetadata["Exif.Photo.DateTimeDigitized"] = exifdatetime;
+            d->exifMetadata()["Exif.Photo.DateTimeDigitized"] = exifdatetime;
 
 #ifdef _XMP_SUPPORT_
 
@@ -708,17 +723,17 @@ bool KExiv2::setImageDateTime(const QDateTime& dateTime, bool setDateTimeDigitiz
 
         Exiv2::Value::AutoPtr xmpTxtVal = Exiv2::Value::create(Exiv2::xmpText);
         xmpTxtVal->read(xmpdatetime);
-        d->xmpMetadata.add(Exiv2::XmpKey("Xmp.exif.DateTimeOriginal"), xmpTxtVal.get());
-        d->xmpMetadata.add(Exiv2::XmpKey("Xmp.photoshop.DateCreated"), xmpTxtVal.get());
-        d->xmpMetadata.add(Exiv2::XmpKey("Xmp.tiff.DateTime"),         xmpTxtVal.get());
-        d->xmpMetadata.add(Exiv2::XmpKey("Xmp.xmp.CreateDate"),        xmpTxtVal.get());
-        d->xmpMetadata.add(Exiv2::XmpKey("Xmp.xmp.MetadataDate"),      xmpTxtVal.get());
-        d->xmpMetadata.add(Exiv2::XmpKey("Xmp.xmp.ModifyDate"),        xmpTxtVal.get());
+        d->xmpMetadata().add(Exiv2::XmpKey("Xmp.exif.DateTimeOriginal"), xmpTxtVal.get());
+        d->xmpMetadata().add(Exiv2::XmpKey("Xmp.photoshop.DateCreated"), xmpTxtVal.get());
+        d->xmpMetadata().add(Exiv2::XmpKey("Xmp.tiff.DateTime"),         xmpTxtVal.get());
+        d->xmpMetadata().add(Exiv2::XmpKey("Xmp.xmp.CreateDate"),        xmpTxtVal.get());
+        d->xmpMetadata().add(Exiv2::XmpKey("Xmp.xmp.MetadataDate"),      xmpTxtVal.get());
+        d->xmpMetadata().add(Exiv2::XmpKey("Xmp.xmp.ModifyDate"),        xmpTxtVal.get());
         if(setDateTimeDigitized)
-            d->xmpMetadata.add(Exiv2::XmpKey("Xmp.exif.DateTimeDigitized"), xmpTxtVal.get());
+            d->xmpMetadata().add(Exiv2::XmpKey("Xmp.exif.DateTimeDigitized"), xmpTxtVal.get());
 
         // Tag not updated:
-        // "Xmp.dc.DateTime" is a sequence of date relevant of dublin core change. 
+        // "Xmp.dc.DateTime" is a sequence of date relevant of dublin core change.
         //                   This is not the picture date as well
 
 #endif // _XMP_SUPPORT_
@@ -727,17 +742,17 @@ bool KExiv2::setImageDateTime(const QDateTime& dateTime, bool setDateTimeDigitiz
 
         const std::string &iptcdate(dateTime.date().toString(Qt::ISODate).toAscii().constData());
         const std::string &iptctime(dateTime.time().toString(Qt::ISODate).toAscii().constData());
-        d->iptcMetadata["Iptc.Application2.DateCreated"] = iptcdate;
-        d->iptcMetadata["Iptc.Application2.TimeCreated"] = iptctime;
+        d->iptcMetadata()["Iptc.Application2.DateCreated"] = iptcdate;
+        d->iptcMetadata()["Iptc.Application2.TimeCreated"] = iptctime;
         if(setDateTimeDigitized)
         {
-            d->iptcMetadata["Iptc.Application2.DigitizationDate"] = iptcdate;
-            d->iptcMetadata["Iptc.Application2.DigitizationTime"] = iptctime;
+            d->iptcMetadata()["Iptc.Application2.DigitizationDate"] = iptcdate;
+            d->iptcMetadata()["Iptc.Application2.DigitizationTime"] = iptctime;
         }
 
         return true;
     }
-    catch( Exiv2::Error &e )
+    catch( Exiv2::Error& e )
     {
         d->printExiv2ExceptionError("Cannot set Date & Time into image using Exiv2 ", e);
     }
@@ -751,11 +766,11 @@ QDateTime KExiv2::getDigitizationDateTime(bool fallbackToCreationTime) const
     {
         // In first, trying to get Date & time from Exif tags.
 
-        if (!d->exifMetadata.empty())
+        if (!d->exifMetadata().empty())
         {
             // Try Exif date time digitized.
 
-            Exiv2::ExifData exifData(d->exifMetadata);
+            Exiv2::ExifData exifData(d->exifMetadata());
             Exiv2::ExifKey key("Exif.Photo.DateTimeDigitized");
             Exiv2::ExifData::iterator it = exifData.findKey(key);
 
@@ -765,7 +780,7 @@ QDateTime KExiv2::getDigitizationDateTime(bool fallbackToCreationTime) const
 
                 if (dateTime.isValid())
                 {
-                    kDebug(51003) << "DateTime (Exif digitalized): " << dateTime.toString().toAscii().constData() << endl;
+                    kDebug() << "DateTime (Exif digitalized): " << dateTime.toString().toAscii().constData();
                     return dateTime;
                 }
             }
@@ -774,12 +789,12 @@ QDateTime KExiv2::getDigitizationDateTime(bool fallbackToCreationTime) const
 
         // In second, trying to get Date & time from Iptc tags.
 
-        if (!d->iptcMetadata.empty())
+        if (!d->iptcMetadata().empty())
         {
 
             // Try digitization Iptc date time entries.
 
-            Exiv2::IptcData iptcData(d->iptcMetadata);
+            Exiv2::IptcData iptcData(d->iptcMetadata());
             Exiv2::IptcKey keyDigitizationDate("Iptc.Application2.DigitizationDate");
             Exiv2::IptcData::iterator it = iptcData.findKey(keyDigitizationDate);
 
@@ -800,14 +815,14 @@ QDateTime KExiv2::getDigitizationDateTime(bool fallbackToCreationTime) const
 
                     if (dateTime.isValid())
                     {
-                        kDebug(51003) << "Date (IPTC digitalized): " << dateTime.toString().toAscii().constData() << endl;
+                        kDebug() << "Date (IPTC digitalized): " << dateTime.toString().toAscii().constData();
                         return dateTime;
                     }
                 }
             }
         }
     }
-    catch( Exiv2::Error &e )
+    catch( Exiv2::Error& e )
     {
         d->printExiv2ExceptionError("Cannot parse Exif digitization date & time tag using Exiv2 ", e);
     }
@@ -830,7 +845,7 @@ bool KExiv2::getImagePreview(QImage& preview) const
 
         // TODO : Added here Makernotes preview extraction when Exiv2 will be fixed for that.
     }
-    catch( Exiv2::Error &e )
+    catch( Exiv2::Error& e )
     {
         d->printExiv2ExceptionError("Cannot get image preview using Exiv2 ", e);
     }
@@ -851,20 +866,20 @@ bool KExiv2::setImagePreview(const QImage& preview, bool setProgramName) const
 
         // A little bit compressed preview jpeg image to limit IPTC size.
         preview.save(&buffer, "JPEG");
-        kDebug(51003) << "JPEG image preview size: (" << preview.width() << " x " 
-                      << preview.height() << ") pixels - " << data.size() << " bytes" << endl;
+        kDebug() << "JPEG image preview size: (" << preview.width() << " x "
+                      << preview.height() << ") pixels - " << data.size() << " bytes";
 
         Exiv2::DataValue val;
         val.read((Exiv2::byte *)data.data(), data.size());
-        d->iptcMetadata["Iptc.Application2.Preview"] = val;
+        d->iptcMetadata()["Iptc.Application2.Preview"] = val;
 
         // See http://www.iptc.org/std/IIM/4.1/specification/IIMV4.1.pdf Appendix A for details.
-        d->iptcMetadata["Iptc.Application2.PreviewFormat"]  = 11;  // JPEG 
-        d->iptcMetadata["Iptc.Application2.PreviewVersion"] = 1;
+        d->iptcMetadata()["Iptc.Application2.PreviewFormat"]  = 11;  // JPEG
+        d->iptcMetadata()["Iptc.Application2.PreviewVersion"] = 1;
 
         return true;
     }
-    catch( Exiv2::Error &e )
+    catch( Exiv2::Error& e )
     {
         d->printExiv2ExceptionError("Cannot get image preview using Exiv2 ", e);
     }
