@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2007 by John Layt <john@layt.net>                       *
+ *   Copyright (C) 2007,2010 by John Layt <john@layt.net>                  *
  *                                                                         *
  *   FilePrinterPreview based on KPrintPreview (originally LGPL)           *
  *   Copyright (c) 2007 Alex Merry <huntedhacker@tiscali.co.uk>            *
@@ -78,12 +78,23 @@ int FilePrinter::doPrintFiles( QPrinter &printer, QStringList fileList, FileDele
         QFileInfo inputFileInfo = QFileInfo( fileList[0] );
         QFileInfo outputFileInfo = QFileInfo( printer.outputFileName() );
 
+        bool doDeleteFile = (fileDeletePolicy == FilePrinter::SystemDeletesFiles);
         if ( inputFileInfo.suffix() == outputFileInfo.suffix() ) {
-            int res = QFile::copy( fileList[0], printer.outputFileName() );
-            if ( res ) {
-                ret = 0;
+            if ( doDeleteFile ) {
+                bool res = QFile::rename( fileList[0], printer.outputFileName() );
+                if ( res ) {
+                    doDeleteFile = false;
+                    ret = 0;
+                } else {
+                    ret = -5;
+                }
             } else {
-                ret = -5;
+                bool res = QFile::copy( fileList[0], printer.outputFileName() );
+                if ( res ) {
+                    ret = 0;
+                } else {
+                    ret = -5;
+                }
             }
         } else if ( inputFileInfo.suffix() == "ps" && outputFileInfo.suffix() == "pdf" && ps2pdfAvailable() ) {
             exe = "ps2pdf";
@@ -99,7 +110,7 @@ int FilePrinter::doPrintFiles( QPrinter &printer, QStringList fileList, FileDele
             ret = -5;
         }
 
-        if ( fileDeletePolicy == FilePrinter::SystemDeletesFiles ) {
+        if ( doDeleteFile ) {
             QFile::remove( fileList[0] );
         }
 
@@ -135,6 +146,12 @@ int FilePrinter::doPrintFiles( QPrinter &printer, QStringList fileList, FileDele
 }
 
 QList<int> FilePrinter::pageList( QPrinter &printer, int lastPage, const QList<int> &selectedPageList )
+{
+    return pageList( printer, lastPage, 0, selectedPageList );
+}
+
+QList<int> FilePrinter::pageList( QPrinter &printer, int lastPage,
+                                  int currentPage, const QList<int> &selectedPageList )
 {
     if ( printer.printRange() == QPrinter::Selection) {
         return selectedPageList;
@@ -146,6 +163,11 @@ QList<int> FilePrinter::pageList( QPrinter &printer, int lastPage, const QList<i
     if ( printer.printRange() == QPrinter::PageRange ) {
         startPage = printer.fromPage();
         endPage = printer.toPage();
+#if QT_VERSION >= KDE_MAKE_VERSION(4,7,0)
+    } else if ( printer.printRange() == QPrinter::CurrentPage) {
+        startPage = currentPage;
+        endPage = currentPage;
+#endif
     } else { //AllPages
         startPage = 1;
         endPage = lastPage;
@@ -296,6 +318,43 @@ QSize FilePrinter::psPaperSize( QPrinter &printer )
     return size;
 }
 
+Generator::PrintError FilePrinter::printError( int c )
+{
+    Generator::PrintError pe;
+    if ( c >= 0 )
+    {
+        pe = Generator::NoPrintError;
+    } 
+    else {
+        switch ( c )
+        {
+            case -1:
+                pe = Generator::PrintingProcessCrashPrintError;
+                break;
+            case -2:
+                pe = Generator::PrintingProcessStartPrintError;
+                break;
+            case -5:
+                pe = Generator::PrintToFilePrintError;
+                break;
+            case -6:
+                pe = Generator::InvalidPrinterStatePrintError;
+                break;
+            case -7:
+                pe = Generator::UnableToFindFilePrintError;
+                break;
+            case -8:
+                pe = Generator::NoFileToPrintError;
+                break;
+            case -9:
+                pe = Generator::NoBinaryToPrintError;
+                break;
+            default:
+                pe = Generator::UnknownPrintError;
+        }
+    }
+    return pe;
+}
 
 
 
@@ -601,10 +660,11 @@ QStringList FilePrinter::optionCupsProperties( QPrinter &printer )
         if ( dialogOptions[i+1].isEmpty() ) {
             cupsOptions << "-o" << dialogOptions[i];
         } else {
-            cupsOptions << "-o" << dialogOptions[i] + "=" + dialogOptions[i+1];
+            cupsOptions << "-o" << dialogOptions[i] + '=' + dialogOptions[i+1];
         }
     }
 
     return cupsOptions;
 }
 
+/* kate: replace-tabs on; indent-width 4; */
