@@ -151,6 +151,7 @@ class Part : public KParts::ReadOnlyPart, public Okular::DocumentObserver, publi
         void slotShowProperties();
         void slotShowEmbeddedFiles();
         void slotShowLeftPanel();
+        void slotShowBottomBar();
         void slotShowPresentation();
         void slotHidePresentation();
         void slotExportAs(QAction *);
@@ -161,7 +162,8 @@ class Part : public KParts::ReadOnlyPart, public Okular::DocumentObserver, publi
         void cannotQuit();
         void slotShowFindBar();
         void slotHideFindBar();
-        void setMimeTypes(KIO::Job *job);
+        void slotJobStarted(KIO::Job *job);
+        void slotJobFinished(KJob *job);
         void loadCancelled(const QString &reason);
         void setWindowTitleFromDocument();
         // can be connected to widget elements
@@ -203,6 +205,7 @@ class Part : public KParts::ReadOnlyPart, public Okular::DocumentObserver, publi
         QPointer<PageView> m_pageView;
         QPointer<TOC> m_toc;
         QPointer<MiniBar> m_miniBar;
+        QPointer<QWidget> m_bottomBar;
         QPointer<PresentationWidget> m_presentationWidget;
         QPointer<ProgressWidget> m_progressWidget;
         QPointer<PageSizeLabel> m_pageSizeLabel;
@@ -225,8 +228,8 @@ class Part : public KParts::ReadOnlyPart, public Okular::DocumentObserver, publi
         KAction *m_gotoPage;
         KAction *m_prevPage;
         KAction *m_nextPage;
-        KAction *m_firstPage;
-        KAction *m_lastPage;
+        KAction *m_beginningOfDocument;
+        KAction *m_endOfDocument;
         KAction *m_historyBack;
         KAction *m_historyNext;
         KAction *m_addBookmark;
@@ -248,6 +251,7 @@ class Part : public KParts::ReadOnlyPart, public Okular::DocumentObserver, publi
         KAction *m_showPresentation;
         KToggleAction* m_showMenuBarAction;
         KToggleAction* m_showLeftPanel;
+        KToggleAction* m_showBottomBar;
         KToggleFullScreenAction* m_showFullScreenAction;
         KAction *m_aboutBackend;
         KAction *m_reload;
