@@ -29,6 +29,7 @@
 
 class KAction;
 class KActionCollection;
+class KMenu;
 class KUrl;
 
 namespace Okular {
@@ -66,7 +67,7 @@ Q_OBJECT
         bool canFitPageWidth() const;
         void fitPageWidth( int page );
         // keep in sync with pageviewutils
-        void displayMessage( const QString & message , PageViewMessage::Icon icon=PageViewMessage::Info, int duration=-1 );
+        void displayMessage( const QString & message, const QString & details = QString(), PageViewMessage::Icon icon=PageViewMessage::Info, int duration=-1 );
 
         // inherited from DocumentObserver
         uint observerId() const { return PAGEVIEW_ID; }
@@ -99,17 +100,17 @@ Q_OBJECT
     public slots:
         void errorMessage( const QString & message, int duration = -1 )
         {
-            displayMessage( message, PageViewMessage::Error, duration );
+            displayMessage( message, QString(), PageViewMessage::Error, duration );
         }
 
         void noticeMessage( const QString & message, int duration = -1 )
         {
-            displayMessage( message, PageViewMessage::Info, duration );
+            displayMessage( message, QString(), PageViewMessage::Info, duration );
         }
 
         void warningMessage( const QString & message, int duration = -1 )
         {
-            displayMessage( message, PageViewMessage::Warning, duration );
+            displayMessage( message, QString(), PageViewMessage::Warning, duration );
         }
 
         void copyTextSelection() const;
@@ -174,12 +175,16 @@ Q_OBJECT
 
         void resizeContentArea( const QSize & newSize );
 
+        void addWebShortcutsMenu( KMenu * menu, const QString & text );
+
         // don't want to expose classes in here
         class PageViewPrivate * d;
 
     private slots:
-        // activated either directly or via QTimer on the viewportResizeEvent
+        // activated either directly or via queued connection on notifySetup
         void slotRelayoutPages();
+        // activated by the resize event delay timer
+        void delayedResizeEvent();
         // activated either directly or via the contentsMoving(int,int) signal
         void slotRequestVisiblePixmaps( int newValue = -1 );
         // activated by the viewport move timer
@@ -193,6 +198,9 @@ Q_OBJECT
         // activated by left click timer
         void slotShowSizeAllCursor();
 
+        void slotHandleWebShortcutAction();
+        void slotConfigureWebShortcuts();
+
         // connected to local actions (toolbar, menu, ..)
         void slotZoom();
         void slotZoomIn();
